#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
URL related functions of the userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""


# python standard library:
import logging
import mimetypes
from typing import Dict, Optional, Union
import urllib.parse

from userprovided import err


def is_url(url: str,
           require_specific_schemes: Union[tuple, None] = None) -> bool:
    """Very basic check if the URL fulfills basic conditions ("LGTM").
       Will not try to connect."""
    parsed = urllib.parse.urlparse(url)

    if parsed.scheme == '':
        logging.debug('The URL has no scheme (like http or https)')
        return False
    if require_specific_schemes:
        if parsed.scheme not in require_specific_schemes:
            logging.error('Scheme %s not supported.', parsed.scheme)
            return False

    if parsed.netloc == '':
        logging.debug('URL is missing or malformed.')
        return False

    return True


def normalize_query_part(query: str,
                         drop_keys: Union[list, tuple, set, None] = None) -> str:
    """Normalize the query part (for example '?foo=1&example=2') of an URL:
       * Remove every chunk that has no value assigned.
       * Sort the remaining chunks alphabetically.
       * Do not change queries without key (old implementations).
        The optional drop_keys allows you to remove specific keys
        (for example trackers)."""
    if is_url(query):
        raise ValueError('Provide only the query part to normalize_query_part')

    if '=' not in query:
        # RFC 3986 prescribes a key=value syntax, but some old implementations
        # do not follow that and generate URLs like:
        # https://www.example.com/forums/forumdisplay.php?example-forum
        # In this case the query part is not changed.
        return query

    chunks = query.split('&')
    keep: Dict[str, str] = dict()
    for chunk in chunks:
        if chunk != '' and '=' in chunk:
            split_chunk = chunk.split('=')
            key = split_chunk[0]
            value = split_chunk[1]
            if key != '' and value != '':
                if key in keep:
                    # i.e. we already processed the same key
                    if keep[key] != value:
                        raise err.QueryKeyConflict(
                            'Duplicate URL query key with conflicting values')
                    logging.debug(
                        'Duplicate key in URL query part, but no conflict.')
                elif drop_keys and key in drop_keys:
                    # i.e. the key is in the list of keys to drop
                    pass
                else:
                    keep[key] = value
    ordered = list()
    if keep:
        for key in sorted(keep):
            ordered.append(f"{key}={keep[key]}")

    return '&'.join(ordered) if ordered else ''


def normalize_url(url: str,
                  drop_keys: Union[list, tuple, set, None] = None,
                  do_not_change_query_part: bool = False) -> str:
    """Normalize an URL:
       * remove whitespace around it,
       * convert scheme and hostname to lowercase,
       * remove ports if they are the standard port for the scheme,
       * remove duplicate slashes from the path,
       * remove fragments (like #foo),
       * remove empty elements of the query part,
       * order the elements in the query part by alphabet,
       The optional drop_keys allows you to remove specific keys
       (for example trackers).

       The option do_not_change_query_part is there, because some content
       management systems use duplicate keys with different values. Sometimes
       that must not raise an exception."""
    url = url.strip()

    if not is_url(url):
        raise ValueError('Malformed URL')

    if drop_keys and do_not_change_query_part:
        raise err.ContradictoryParameters(
            'Cannot drop keys AND leave the query part unchanged.')

    # Remove fragments (https://www.example.com#foo -> https://www.example.com)
    url, _ = urllib.parse.urldefrag(url)

    standard_ports = {'http': 80, 'https': 443}

    parsed = urllib.parse.urlparse(url)
    reassemble = list()
    reassemble.append(parsed.scheme.lower())

    if not parsed.port:
        # There is no port to begin with
        # hostname is lowercase without port
        reassemble.append(parsed.hostname)  # type: ignore[arg-type]
    elif (parsed.scheme in standard_ports and
            parsed.port == standard_ports[parsed.scheme]):
        # There is a port and it equals the standard.
        # That means it is redundant.
        reassemble.append(parsed.hostname)  # type: ignore[arg-type]
    else:
        # There is a port but it is not in the list or not standard
        reassemble.append(f"{parsed.hostname}:{parsed.port}")

    # remove common typo (// in path element):
    reassemble.append(parsed.path.replace('//', '/'))

    # do not change parameters of the path element (!= query)
    reassemble.append(parsed.params)

    if do_not_change_query_part:
        reassemble.append(parsed.query)
    else:
        reassemble.append(normalize_query_part(parsed.query, drop_keys))

    # urlunparse expects a fifth element (the already removed fragment)
    reassemble.append('')

    url = urllib.parse.urlunparse(reassemble)

    return url


def determine_file_extension(url: str,
                             provided_mime_type: Optional[str] = None) -> str:
    """Guess the correct filename extension from an URL and / or
    the mime-type returned by the server.
    Sometimes a valid URL does not contain a file extension
    (like https://www.example.com/), or it is ambiguous.
    So the mime type acts as a fallback. In case the correct
    extension cannot be determined at all it is set to 'unknown'."""
    if provided_mime_type:
        provided_mime_type = provided_mime_type.strip()
    if provided_mime_type == '':
        provided_mime_type = None

    extension: Optional[str] = None
    type_by_url: Optional[str] = None
    parsed_url = urllib.parse.urlparse(url)
    if parsed_url.path not in ('', '/'):
        type_by_url = mimetypes.guess_type(parsed_url.path)[0]

    if type_by_url is not None and type_by_url == provided_mime_type:
        # Best case: URL and server header suggest the same filetype.
        extension = mimetypes.guess_extension(provided_mime_type)
    elif type_by_url is None and provided_mime_type is not None:
        # The URL does not contain an usable extension, but
        # the server provides a mime type.
        extension = mimetypes.guess_extension(provided_mime_type)
        if extension is None:
            logging.error('No hint in URL and mime-type malformed for %s', url)
            return '.unknown'
    elif type_by_url is not None and provided_mime_type is None:
        # There is a usable file extension in the URL, but the misconfigured
        # server does not provide a mime type.
        extension = mimetypes.guess_extension(type_by_url)
        # Here no code for extension is None, because mimetypes already
        # guessed a type once we got here and can guess a matching extension.
    elif type_by_url is None and provided_mime_type is None:
        # Neither the URL nor the server does hint to a extension
        msg = (f"Neither URL ({url}) nor mime-type ({provided_mime_type}) " +
               "suggests a file extension.")
        logging.error(msg)
        return '.unknown'
    elif type_by_url != provided_mime_type:
        # The suggestions contradict each other
        msg = (f"The mime type ({type_by_url}) suggested by the URL ({url}) " +
               "does not match the mime type supplied by the server " +
               f"({provided_mime_type}). Using the extension suggested " +
               "by the URL.")
        logging.error(msg)
        extension = mimetypes.guess_extension(type_by_url)  # type: ignore[arg-type]

    # Handle errors and irregularities in mimetypes:
    if extension == '.bat' and provided_mime_type == 'text/plain':
        # text/plain is mapped to .bat in python 3.6.
        # Python 3.8 correctly guesses .txt as extension.
        return '.txt'

    if extension == '.htm':
        return '.html'

    if extension is None:
        return '.unknown'

    return extension



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Checking and normalizing dates for the userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""


import datetime
import logging
import re


def date_exists(year: int,
                month: int,
                day: int) -> bool:
    """Check if a date given by three integers is valid
       i.e exists in the calendar. """
    try:
        # int() will convert something like '01' to 1
        year = int(year)
        month = int(month)
        day = int(day)
    except ValueError:
        logging.error('Could not convert date parts to integer.')
        return False

    try:
        datetime.datetime(year, month, day)
    except ValueError:
        logging.error('Provided date does not exist in the calendar.')
        return False
    return True


def date_en_long_to_iso(date_string: str) -> str:
    """ Take a long format English date and return a standardized date string
       (i.e. YYYY-MM-DD). """
    date_string = date_string.strip()
    regex_long_date_en = re.compile(
        r"(?P<monthL>[a-zA-Z\.]{3,9})\s+(?P<day>\d{1,2})(th)?,\s*(?P<year>\d\d\d\d)")
    try:
        match = re.search(regex_long_date_en, date_string)
        if match:
            match_year = match.group('year')
            match_month = match.group('monthL')
            match_day = match.group('day')
        else:
            raise AttributeError('No date provided')
    except AttributeError:
        logging.error('Malformed date')
        raise

    # add a zero to day if <10
    if len(match_day) == 1:
        match_day = '0' + match_day
    months = {
        'January': '01', 'Jan.': '01',
        'February': '02', 'Feb.': '02',
        'March': '03', 'Mar.': '03',
        'April': '04', 'Apr.': '04',
        'May': '05',
        'June': '06', 'Jun.': '06',
        'July': '07', 'Jul.': '07',
        'August': '08', 'Aug.': '08',
        'September': '09', 'Sep.': '09',
        'October': '10', 'Oct.': '10',
        'November': '11', 'Nov.': '11',
        'December': '12', 'Dec.': '12'
        }
    try:
        match_month = months[str(match_month).lower().capitalize()]
    except KeyError:
        # String for month matched the regular expression but is no
        # recognized month.
        logging.error('Do not recognize month.')
        raise

    if not date_exists(int(match_year), int(match_month), int(match_day)):
        raise ValueError('Provided date is invalid.')

    return f"{match_year}-{match_month}-{match_day}"


def date_de_long_to_iso(date_string: str) -> str:
    """Take a long format German date and return a standardized date string
       (i.e. YYYY-MM-DD). """
    date_string = date_string.strip()
    regex_long_date_de = re.compile(
        r"(?P<day>\d{1,2})\.\s+(?P<monthL>[a-zA-Z\.]{3,9})\s+(?P<year>\d\d\d\d)")
    try:
        match = re.search(regex_long_date_de, date_string)
        if match:
            match_year = match.group('year')
            match_month = match.group('monthL')
            match_day = match.group('day')
        else:
            raise AttributeError('No date provided')
    except AttributeError:
        logging.exception('Malformed date')
        raise

    # add a zero to day if <10
    if len(match_day) == 1:
        match_day = '0' + match_day
    months = {
        'Januar': '01', 'Jan.': '01',
        'Februar': '02', 'Feb.': '02',
        'März': '03', 'Mar.': '03',
        'April': '04', 'Apr.': '04',
        'Mai': '05',
        'Juni': '06', 'Jun.': '06',
        'Juli': '07', 'Jul.': '07',
        'August': '08', 'Aug.': '08',
        'September': '09', 'Sep.': '09',
        'Oktober': '10', 'Okt.': '10',
        'November': '11', 'Nov.': '11',
        'Dezember': '12', 'Dez.': '12'
        }
    try:
        match_month = months[str(match_month).lower().capitalize()]
    except KeyError:
        # String for month matched the regular expression but is no
        # recognized month.
        logging.exception('Do not recognize month.')
        raise

    if not date_exists(int(match_year), int(match_month), int(match_day)):
        raise ValueError('Provided date is invalid.')

    return f"{match_year}-{match_month}-{match_day}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Userprovided: Custom Exceptions

Source: https://github.com/RuedigerVoigt/userprovided
(c) 2019-2021 Rüdiger Voigt:
Released under the Apache License 2.0
"""


class UserprovidedException(Exception):
    "An exception specific to userprovided occured"
    def __init__(self, *args, **kwargs):  # type: ignore[no-untyped-def]
        Exception.__init__(self, *args, **kwargs)


class QueryKeyConflict(UserprovidedException):
    """Within the query part of the same URL a key is duplicate
       AND the values are not identical."""


class DeprecatedHashAlgorithm(UserprovidedException):
    """If you try to use a deprecated hashing algorithm like MD5 or SHA1,
       this exception is thrown."""


class ContradictoryParameters(UserprovidedException, ValueError):
    """Thrown if the user tries to use settings in a mutually exclusive way."""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""

from userprovided import _version
from userprovided import date
from userprovided import hash  # Deprecated / logs warning until v 1.0
from userprovided import hashing
from userprovided import mail
from userprovided import parameters
from userprovided import url


NAME = "userprovided"
__version__ = _version.__version__
__author__ = "Rüdiger Voigt"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Check Parameters
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""

import logging
import re
from typing import Optional, Union

from userprovided import err


def convert_to_set(convert_this: Union[list, set, str, tuple]) -> set:
    """ Convert a string, a tuple, or a list into a set
        (i.e. no duplicates, unordered)"""

    if isinstance(convert_this, set):
        # functions using this expect a set, so everything
        # else just captures bad input by users
        new_set = convert_this
    elif isinstance(convert_this, str):
        new_set = {convert_this}
    elif isinstance(convert_this, list):
        new_set = set(convert_this)
    elif isinstance(convert_this, tuple):
        new_set = set(convert_this)
    else:
        raise TypeError('The function calling this expects a set.')

    return new_set


def validate_dict_keys(dict_to_check: dict,
                       allowed_keys: set,
                       necessary_keys: Optional[set] = None,
                       dict_name: Optional[str] = None) -> bool:
    """If you use dictionaries to pass parameters, there are two common errors:
       * misspelled keys
       * necessary keys are missing
       This functions checks whether all keys are in the set of allowed_keys
       and raises ValueError if a unknown key is found.
       It can also check whether all necessary keys are present and
       raises ValueError if not.
       dict_name can be used for a better error message."""

    if not dict_name:
        # fallback to neutral
        dict_name = 'dictionary'

    # In case something other than a set is provided:
    allowed_keys = convert_to_set(allowed_keys)

    if necessary_keys:
        # also make sure it is a set:
        necessary_keys = convert_to_set(necessary_keys)
        # Are all necessary keys in the allowed key list?
        if len(necessary_keys - allowed_keys) != 0:
            msg = ("Contradiction: Not all necessary keys " +
                   "are in the allowed keys set!")
            logging.exception(msg)
            raise ValueError(msg)

    # Get all keys in the dictionary:
    try:
        found_keys = dict_to_check.keys()
    except AttributeError as no_dict:
        raise AttributeError('Expected a dictionary for the dict_to_check ' +
                             'parameter!') from no_dict

    # Check for unknown keys:
    for key in found_keys:
        if key not in allowed_keys:
            msg = f"Unknown key {key} in {dict_name}"
            logging.exception(msg)
            raise ValueError(msg)
    logging.debug('No unknown keys found.')

    # Check if all necessary keys are present:
    if necessary_keys:
        for key in necessary_keys:
            if key not in found_keys:
                msg = f"Necessary key {key} missing in {dict_name}!"
                logging.exception(msg)
                raise ValueError(msg)
        logging.debug('All necessary keys found.')

    return True


def keys_neither_none_nor_empty(dict_to_check: dict) -> bool:
    """Checks if all keys in the provided dictionary are neither None, nor
       have an empty value (like an empty string (including whitespace only)
       or a dict/list/set/tuple without elements).

       Does ignore value types other than dict/list/set/str/tuple.

       Raises Value Error if you provide something else than a dictionary,
       or if its is completely empty."""

    if not isinstance(dict_to_check, dict):
        raise ValueError('This is not a dictionary')
    if len(dict_to_check) == 0:
        raise ValueError('This dictionary is empty')

    def error_found() -> None:
        logging.error("Dictionary contains key that is either empty or None!")

    for _, value in dict_to_check.items():
        if value is None:
            error_found()
            return False
        if isinstance(value, str):
            if len(value.strip()) == 0:
                error_found()
                return False
        if isinstance(value, (dict, list, set, tuple)):
            if len(value) == 0:
                error_found()
                return False

    return True


def numeric_in_range(parameter_name: str,
                     given_value: Union[int, float],
                     minimum_value: Union[int, float],
                     maximum_value: Union[int, float],
                     fallback_value: Union[int, float]
                     ) -> Union[int, float]:
    """Checks if a numeric value is within a specified range.
       If not this returns the fallback value and logs a warning."""
    if not parameter_name:
        parameter_name = ''

    for param in {given_value, minimum_value, maximum_value, fallback_value}:
        if not isinstance(param, (int, float)):
            raise ValueError('Value must be numeric.')

    if minimum_value > maximum_value:
        raise err.ContradictoryParameters(
            "Minimum must not be larger than maximum value.")

    if fallback_value < minimum_value or fallback_value > maximum_value:
        raise err.ContradictoryParameters(
            "Fallback value outside the allowed range.")

    if given_value < minimum_value:
        msg = (f"Value of {parameter_name} is below the minimum allowed." +
               f"Falling back to {fallback_value}.")
        logging.warning(msg)
        return fallback_value

    if given_value > maximum_value:
        msg = (f"Value of {parameter_name} is above the maximum allowed." +
               f"Falling back to {fallback_value}.")
        logging.warning(msg)
        return fallback_value

    # passed all checks:
    return given_value


def int_in_range(parameter_name: str,
                 given_value: int,
                 minimum_value: int,
                 maximum_value: int,
                 fallback_value: int) -> int:
    """Special case of numeric_in_range: check if given integer is
       within a specified range of possible values."""
    for param in {given_value, minimum_value, maximum_value, fallback_value}:
        if type(param) != int:  # pylint: disable=unidiomatic-typecheck
            raise ValueError('Value must be an integer.')
    return int(numeric_in_range(parameter_name,
                                given_value,
                                minimum_value,
                                maximum_value,
                                fallback_value))


def is_port(port_number: int) -> bool:
    """Check if the number provided is valid as a TCP/UDP port
       (i.e an integer in the range from 0 to 65535)."""

    if not isinstance(port_number, int):
        raise ValueError('Port has to be an integer.')

    if 0 < port_number < 65536:
        logging.debug('Port within range')
        return True
    logging.error('Port not within valid range from 0 to 65535')
    return False


def string_in_range(string_to_check: str,
                    minimum_length: int,
                    maximum_lenght: int,
                    strip_string: bool = True) -> bool:
    """Strips whitespace from both ends of a string and then checks
       if the length of that string falls in those limits.
       The strip() can be turned off. """

    if minimum_length > maximum_lenght:
        raise err.ContradictoryParameters("Minimum must not be larger than maximum value.")
    enforce_boolean(strip_string)

    if strip_string:
        string_to_check = string_to_check.strip()
    if len(string_to_check) < minimum_length:
        logging.info("String length below minimum length.")
        return False
    if len(string_to_check) > maximum_lenght:
        logging.info("String longer than maximum.")
        return False
    return True


def is_aws_s3_bucket_name(bucket_name: str) -> bool:
    """Returns True if bucket name is well-formed for AWS S3 buckets

    Applying the rules set here:
    https://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html
    """

    # Lengthy code which could be written as a single regular expression.
    # However written in this way to provide useful error messages.
    if len(bucket_name) < 3:
        logging.error(
            'Any AWS bucket name has to be at least 3 characters long.')
        return False
    if len(bucket_name) > 63:
        logging.error(
            'The AWS bucket name exceeds the maximum length of 63 characters.')
        return False
    if not re.match(r"^[a-z0-9\-\.]*$", bucket_name):
        logging.error('The AWS bucket name contains invalid characters.')
        return False
    if re.match(r"[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}",
                bucket_name):
        # Check if the bucket name resembles an IPv4 address.
        # No need to check IPv6 as the colon is not an allowed character.
        logging.error('An AWS must not resemble an IP address.')
        return False
    if re.match(r"([a-z0-9][a-z0-9\-]*[a-z0-9]\.)*[a-z0-9][a-z0-9\-]*[a-z0-9]",
                bucket_name):
        # Must start with a lowercase letter or number
        # Bucket names must be a series of one or more labels.
        # Adjacent labels are separated by a single period (.).
        # Each label must start and end with a lowercase letter or a number.
        # => Adopted the answer provided by Zak (zero or more labels
        # followed by a dot) found here:
        # https://stackoverflow.com/questions/50480924
        return True

    logging.error('Invalid AWS bucket name.')
    return False


def enforce_boolean(parameter_value: bool,
                    parameter_name: Optional[str] = None) -> None:
    """Raise a ValueError if the parameter is not of type bool."""
    if type(parameter_value) != bool:  # pylint: disable=unidiomatic-typecheck
        parameter_name = 'parameter' if parameter_name else ''
        raise ValueError(f"Value of {parameter_name} must be boolean," +
                         "i.e True / False (without quotation marks).")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Hash functionality for the userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""


import hashlib
import logging
import pathlib
from typing import Optional, Union

from userprovided import err


def hash_available(hash_method: str,
                   fail_on_deprecated: bool = True) -> bool:
    """Checks if the supplied hashing algorithm is available.
       If fail_on_deprecated is set to True (default) it will raise an
       exception if the method is deprecated (md5 and sha1)."""

    if hash_method:
        hash_method = hash_method.strip()

    if hash_method == '' or hash_method is None:
        raise ValueError('No hash method provided')

    # Is the chosen method available and supported?
    hash_method = hash_method.strip()

    if fail_on_deprecated:
        if hash_method in ('md5', 'sha1'):
            raise err.DeprecatedHashAlgorithm('The supplied hash method %s is deprecated!')

    if hash_method in hashlib.algorithms_available:
        logging.debug('Hash method %s is available.', hash_method)
        return True
    return False


def calculate_file_hash(file_path: Union[pathlib.Path, str],
                        hash_method: str = 'sha256',
                        expected_hash: Optional[str] = None) -> str:
    """Calculate hash value for a file.
       Supported: SHA224 / SHA256 / SHA512
       If you provide expected_hash this will raise a ValueError exception
       in case this does not match the calculated hash. This allows you to
       detect changes or tampering."""

    if hash_method in ('md5', 'sha1'):
        raise err.DeprecatedHashAlgorithm(
            'Deprecated hash method not supported')

    if hash_available(hash_method):
        if hash_method == 'sha224':
            h = hashlib.sha224()
        elif hash_method == 'sha256':
            h = hashlib.sha256()
        elif hash_method == 'sha512':
            h = hashlib.sha512()
        else:
            raise ValueError('Hash method not supported')
    else:
        raise ValueError(f"Hash method {hash_method} not available on system.")

    try:
        with open(pathlib.Path(file_path), 'rb') as file:
            content = file.read()
        h.update(content)
        calculated_hash = h.hexdigest()
        if expected_hash and expected_hash != calculated_hash:
            mismatch_message = ("Mismatch between calculated and expected " +
                                f"{hash_method} hash for {file_path}")
            logging.exception(mismatch_message)
            raise ValueError(mismatch_message)
        return calculated_hash
    except FileNotFoundError:
        logging.exception(
            'Cannot calculate hash: File not found or not readable.',
            exc_info=True)
        raise
    except PermissionError:
        logging.exception(
            'Cannot calculate file hash: insufficient permissions.',
            exc_info=True)
        raise
    except Exception:
        logging.error('Exception while trying to get file hash',
                      exc_info=True)
        raise



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Hash functionality for the userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""

import logging
import pathlib
from typing import Optional, Union
import warnings

from userprovided import hashing


def hash_available(hash_method: str,
                   fail_on_deprecated: bool = True) -> bool:
    """DEPRECATED: use userprovided.hashing.hash_available instead.
       This will fail with userprovied version 1.0.0"""
    msg = 'Deprecated call: method moved to userprovided.hashing'
    logging.warning(msg)
    warnings.warn(msg, DeprecationWarning)
    return hashing.hash_available(hash_method, fail_on_deprecated)


def calculate_file_hash(file_path: Union[pathlib.Path, str],
                        hash_method: str = 'sha256',
                        expected_hash: Optional[str] = None) -> str:
    """DEPRECATED: use userprovided.hashing.calculate_file_hash instead.
       This will fail with userprovied version 1.0.0"""
    msg = 'Deprecated call: method moved to userprovided.hashing'
    logging.warning(msg)
    warnings.warn(msg, DeprecationWarning)
    return hashing.calculate_file_hash(file_path, hash_method, expected_hash)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Checking Email for the userprovided library
~~~~~~~~~~~~~~~~~~~~~
Source: https://github.com/RuedigerVoigt/userprovided
(c) 2020-2021 Rüdiger Voigt
Released under the Apache License 2.0
"""

# python standard library:
import logging
import re


def is_email(mailaddress: str) -> bool:
    "Very basic check if the email address has a valid format."

    if not mailaddress or mailaddress == '':
        logging.warning('No mail address supplied.')
        return False

    mailaddress = mailaddress.strip()
    if not re.match(r"^[^\s@]+@[^\s@]+\.[a-zA-Z]+", mailaddress):
        logging.error(
            'The supplied mailaddress %s has an unknown format.', mailaddress)
        return False

    logging.debug('%s seems to have a valid format', mailaddress)
    return True
